@@ -247,6 +247,22 @@ pub fn cli<'a>() -> App<'a> {
                     .value_name("LIMIT")
                     .about("Only list LIMIT submits")
                 )
+                .arg(Arg::new("for-commit")
+                    .required(false)
+                    .multiple(false)
+                    .long("commit")
+                    .takes_value(true)
+                    .value_name("HASH")
+                    .about("Limit listed submits to one commit hash")
+                )
+                .arg(Arg::new("image")
+                    .required(false)
+                    .multiple(false)
+                    .long("image")
+                    .takes_value(true)
+                    .value_name("IMAGE")
+                    .about("Limit listed submits to submits on IMAGE")
+                )
             )
 
             .subcommand(App::new("jobs")
@@ -646,6 +662,15 @@ pub fn cli<'a>() -> App<'a> {
                 .validator(env_pass_validator)
                 .about("Filter for this \"key=value\" environment variable")
             )
+            .arg(Arg::new("image")
+                .required(false)
+                .multiple(false)
+                .long("image")
+                .short('I')
+                .takes_value(true)
+                .value_name("IMAGE")
+                .about("Only list artifacts that were built on IMAGE")
+            )
         )
 
         .subcommand(App::new("find-pkg")
@@ -795,6 +820,20 @@ pub fn cli<'a>() -> App<'a> {
                     .value_name("VERSION")
                     .about("Verify the sources of this package version (optional, if left out, all packages are checked)")
                 )
+
+                .arg(Arg::new("matching")
+                    .required(false)
+                    .multiple(false)
+                    .long("matching")
+                    .takes_value(true)
+                    .value_name("REGEX")
+                    .about("Verify all packages where the package name matches REGEX")
+                )
+
+                .group(ArgGroup::new("verify-one-or-many")
+                    .args(&["package_name", "matching"])
+                    .required(true)
+                )
             )
             .subcommand(App::new("list-missing")
                 .version(crate_version!())
@@ -826,7 +865,7 @@ pub fn cli<'a>() -> App<'a> {
                     .multiple(false)
                     .index(1)
                     .value_name("PKG")
-                    .about("Download the sources of this package (optional, if left out, all packages are downloaded)")
+                    .about("Download the sources of this package")
                 )
                 .arg(Arg::new("package_version")
                     .required(false)
@@ -841,6 +880,20 @@ pub fn cli<'a>() -> App<'a> {
                     .long("force")
                     .about("Overwrite existing cache entry")
                 )
+
+                .arg(Arg::new("matching")
+                    .required(false)
+                    .multiple(false)
+                    .long("matching")
+                    .takes_value(true)
+                    .value_name("REGEX")
+                    .about("Download all packages matching a regex with their name")
+                )
+
+                .group(ArgGroup::new("download-one-or-many")
+                    .args(&["package_name", "matching"])
+                    .required(true)
+                )
             )
             .subcommand(App::new("of")
                 .version(crate_version!())
@@ -1008,6 +1061,35 @@ pub fn cli<'a>() -> App<'a> {
                 .value_name("VERSION_CONSTRAINT")
                 .about("A version constraint to search for (optional), E.G. '=1.0.0'")
             )
+            .arg(Arg::new("image")
+                .required(false)
+                .multiple(false)
+                .takes_value(true)
+                .value_name("IMAGE NAME")
+                .short('I')
+                .long("image")
+                .about("Name of the docker image to use")
+                .long_about(indoc::indoc!(r#"
+                    Name of the docker image to use.
+
+                    Required because tree might look different on different images because of
+                    conditions on dependencies.
+                "#))
+            )
+            .arg(Arg::new("env")
+                .required(false)
+                .multiple(true)
+                .short('E')
+                .long("env")
+                .validator(env_pass_validator)
+                .about("Additional env to be passed when building packages")
+                .long_about(indoc::indoc!(r#"
+                    Additional env to be passed when building packages.
+
+                    Required because tree might look different on different images because of
+                    conditions on dependencies.
+                "#))
+            )
         )
 
         .subcommand(App::new("metrics")
@@ -1214,6 +1296,32 @@ pub fn cli<'a>() -> App<'a> {
                     .long_about("Display details about the container. Do not assume the output format to be stable.")
                 )
             )
+            .subcommand(App::new("images")
+                .version(crate_version!())
+                .about("Query images on endpoint(s)")
+                .subcommand(App::new("list")
+                    .version(crate_version!())
+                    .about("List images on endpoint(s)")
+                    .arg(Arg::new("csv")
+                        .required(false)
+                        .multiple(false)
+                        .long("csv")
+                        .takes_value(false)
+                        .about("List top output as CSV")
+                    )
+                )
+                .subcommand(App::new("verify-present")
+                    .version(crate_version!())
+                    .about("Verify that all configured images are present on endpoint(s)")
+                    .arg(Arg::new("csv")
+                        .required(false)
+                        .multiple(false)
+                        .long("csv")
+                        .takes_value(false)
+                        .about("List top output as CSV")
+                    )
+                )
+            )
         )
 }
 
