@@ -33,22 +33,23 @@ use tokio_stream::StreamExt;
 use uuid::Uuid;
 
 use crate::config::*;
-use crate::filestore::path::StoreRoot;
 use crate::filestore::ReleaseStore;
 use crate::filestore::StagingStore;
+use crate::filestore::path::StoreRoot;
 use crate::job::JobResource;
 use crate::log::LogItem;
 use crate::orchestrator::OrchestratorSetup;
+use crate::package::Dag;
 use crate::package::PackageName;
 use crate::package::PackageVersion;
 use crate::package::Shebang;
-use crate::package::Dag;
+use crate::package::condition::ConditionData;
 use crate::repository::Repository;
 use crate::schema;
 use crate::source::SourceCache;
+use crate::util::EnvironmentVariableName;
 use crate::util::docker::ImageName;
 use crate::util::progress::ProgressBars;
-use crate::util::EnvironmentVariableName;
 
 /// Implementation of the "build" subcommand
 #[allow(clippy::too_many_arguments)]
@@ -226,7 +227,12 @@ pub async fn build(
 
     let dag = {
         let bar_tree_building = progressbars.bar();
-        let dag = Dag::for_root_package(package.clone(), &repo, &bar_tree_building)?;
+        let condition_data = ConditionData {
+            image_name: Some(&image_name),
+            env: &additional_env,
+        };
+
+        let dag = Dag::for_root_package(package.clone(), &repo, Some(&bar_tree_building), &condition_data)?;
         bar_tree_building.finish_with_message("Finished loading Dag");
         dag
     };
