@@ -258,6 +258,9 @@ fn submit(conn_cfg: DbConnectionConfig<'_>, matches: &ArgMatches) -> Result<()>
     let submit = models::Submit::with_id(&conn, &submit_id)
         .with_context(|| anyhow!("Loading submit '{}' from DB", submit_id))?;
 
+    let githash = models::GitHash::with_id(&conn, submit.repo_hash_id)
+        .with_context(|| anyhow!("Loading GitHash '{}' from DB", submit.repo_hash_id))?;
+
     let jobs = schema::submits::table
         .inner_join(schema::jobs::table)
         .filter(schema::submits::uuid.eq(&submit_id))
@@ -288,6 +291,7 @@ fn submit(conn_cfg: DbConnectionConfig<'_>, matches: &ArgMatches) -> Result<()>
     indoc::writedoc!(outlock, r#"
             Submit   {submit_id}
             Date:    {submit_dt}
+            Commit:  {submit_commit}
             Jobs:    {n_jobs}
             Success: {n_jobs_success}
             Unknown: {n_jobs_unknown}
@@ -296,6 +300,7 @@ fn submit(conn_cfg: DbConnectionConfig<'_>, matches: &ArgMatches) -> Result<()>
         "#,
         submit_id = submit.uuid.to_string().cyan(),
         submit_dt = submit.submit_time.to_string().cyan(),
+        submit_commit = githash.hash.cyan(),
         n_jobs = n_jobs.to_string().cyan(),
         n_jobs_success = jobs_success.to_string().green(),
         n_jobs_unknown = jobs_unknown.to_string().red(),
@@ -334,54 +339,97 @@ fn submit(conn_cfg: DbConnectionConfig<'_>, matches: &ArgMatches) -> Result<()>
 fn submits(conn_cfg: DbConnectionConfig<'_>, matches: &ArgMatches) -> Result<()> {
     let csv = matches.is_present("csv");
     let limit = matches.value_of("limit").map(i64::from_str).transpose()?;
-    let hdrs = crate::commands::util::mk_header(vec!["Time", "UUID"]);
+    let hdrs = crate::commands::util::mk_header(vec!["Time", "UUID", "For Package", "For Package Version"]);
     let conn = conn_cfg.establish_connection()?;
+    let commit = matches.value_of("for-commit");
 
     let query = schema::submits::table
-        .order_by(schema::submits::id.desc()); // required for the --limit implementation
+        .order_by(schema::submits::id.desc()) // required for the --limit implementation
+        .inner_join(schema::githashes::table.on(schema::submits::repo_hash_id.eq(schema::githashes::id)))
+        .inner_join(schema::images::table)
+        .into_boxed();
+
+    let query = if let Some(commithash) = commit.as_ref() {
+        query.filter(schema::githashes::hash.eq(commithash))
+    } else {
+        query
+    };
+
+    let query = if let Some(image) = matches.value_of("image") {
+        query.filter(schema::images::name.eq(image))
+    } else {
+        query
+    };
 
     let submits = if let Some(pkgname) = matches.value_of("with_pkg").map(String::from) {
+        // In the case of a with_pkg command, we must execute two queries on the database, as the
+        // diesel framework does not yet support aliases for queries (see
+        // https://github.com/diesel-rs/diesel/pull/2254).
+        // This is due to the fact that we need to join the packages table twice, once to filter
+        // out all submits that did not include the "with pkg" and once to join the requested
+        // package for the output.
+
         // Get all submits which included the package, but were not necessarily made _for_ the package
         let query = query
             .inner_join(schema::jobs::table)
             .inner_join(schema::packages::table.on(schema::jobs::package_id.eq(schema::packages::id)))
-            .filter(schema::packages::name.eq(&pkgname))
-            .into_boxed();
+            .filter(schema::packages::name.eq(&pkgname));
 
-        if let Some(limit) = limit {
+        let query = if let Some(limit) = limit {
             query.limit(limit)
         } else {
             query
-        }
-        .select(schema::submits::all_columns)
-        .load::<models::Submit>(&conn)?
+        };
+
+        // Only load the IDs of the submits, so we can later use them to filter the submits
+        let submit_ids = query.select(schema::submits::id).load::<i32>(&conn)?;
+
+        schema::submits::table
+            .order_by(schema::submits::id.desc()) // required for the --limit implementation
+            .inner_join({
+                schema::packages::table.on(schema::submits::requested_package_id.eq(schema::packages::id))
+            })
+            .filter(schema::submits::id.eq_any(submit_ids))
+            .select((schema::submits::all_columns, schema::packages::all_columns))
+            .load::<(models::Submit, models::Package)>(&conn)?
     } else if let Some(pkgname) = matches.value_of("for_pkg") {
         // Get all submits _for_ the package
         let query = query
             .inner_join({
                 schema::packages::table.on(schema::submits::requested_package_id.eq(schema::packages::id))
             })
-            .filter(schema::packages::dsl::name.eq(&pkgname))
-            .into_boxed();
+            .filter(schema::packages::dsl::name.eq(&pkgname));
 
         if let Some(limit) = limit {
             query.limit(limit)
         } else {
             query
         }
-        .select(schema::submits::all_columns)
-        .load::<models::Submit>(&conn)?
+        .select((schema::submits::all_columns, schema::packages::all_columns))
+        .load::<(models::Submit, models::Package)>(&conn)?
     } else if let Some(limit) = limit {
-        query.limit(limit).load::<models::Submit>(&conn)?
+        query
+            .inner_join({
+                schema::packages::table.on(schema::submits::requested_package_id.eq(schema::packages::id))
+            })
+            .select((schema::submits::all_columns, schema::packages::all_columns))
+            .limit(limit)
+            .load::<(models::Submit, models::Package)>(&conn)?
     } else {
-        query.load::<models::Submit>(&conn)?
+        query.inner_join({
+                schema::packages::table.on(schema::submits::requested_package_id.eq(schema::packages::id))
+            })
+            .select((schema::submits::all_columns, schema::packages::all_columns))
+            .load::<(models::Submit, models::Package)>(&conn)?
     };
 
-    // Helper to map Submit -> Vec<String>
-    let submit_to_vec = |submit: models::Submit| {
+    // Helper to map (Submit, Package) -> Vec<String>
+    let submit_to_vec = |(submit, package): (models::Submit, models::Package)| {
         vec![
             submit.submit_time.to_string(),
             submit.uuid.to_string(),
+            package.name,
+            package.version,
         ]
     };
 
