@@ -66,14 +66,23 @@ pub async fn verify(
         .map(PackageVersionConstraint::try_from)
         .transpose()?;
 
+    let matching_regexp = matches.value_of("matching")
+        .map(crate::commands::util::mk_package_name_regex)
+        .transpose()?;
+
     let packages = repo
         .packages()
-        .filter(|p| pname.as_ref().map(|n| p.name() == n).unwrap_or(true))
         .filter(|p| {
-            pvers
-                .as_ref()
-                .map(|v| v.matches(p.version()))
-                .unwrap_or(true)
+            match (pname.as_ref(), pvers.as_ref(), matching_regexp.as_ref()) {
+                (None, None, None)              => true,
+                (Some(pname), None, None)       => p.name() == pname,
+                (Some(pname), Some(vers), None) => p.name() == pname && vers.matches(p.version()),
+                (None, None, Some(regex))       => regex.is_match(p.name()),
+
+                (_, _, _) => {
+                    panic!("This should not be possible, either we select packages by name and (optionally) version, or by regex.")
+                },
+            }
         })
         .inspect(|p| trace!("Found for verification: {} {}", p.name(), p.version()));
 
@@ -214,6 +223,58 @@ pub async fn download(
     repo: Repository,
     progressbars: ProgressBars,
 ) -> Result<()> {
+    async fn perform_download(source: &SourceEntry, bar: &indicatif::ProgressBar) -> Result<()> {
+        trace!("Creating: {:?}", source);
+        let file = source.create().await.with_context(|| {
+            anyhow!(
+                "Creating source file destination: {}",
+                source.path().display()
+            )
+        })?;
+
+        let mut file = tokio::io::BufWriter::new(file);
+        let client = reqwest::Client::builder()
+            .redirect(reqwest::redirect::Policy::limited(10))
+            .build()
+            .context("Building HTTP client failed")?;
+
+        let request = client.get(source.url().as_ref())
+            .build()
+            .with_context(|| anyhow!("Building request for {} failed", source.url().as_ref()))?;
+
+        let response = match client.execute(request).await {
+            Ok(resp) => resp,
+            Err(e) => {
+                bar.finish_with_message(format!("Failed: {}", source.url()));
+                return Err(e).with_context(|| anyhow!("Downloading '{}'", source.url()))
+            }
+        };
+
+        if let Some(len) = response.content_length() {
+            bar.set_length(len);
+        }
+
+        let mut stream = reqwest::get(source.url().as_ref()).await?.bytes_stream();
+        let mut bytes_written = 0;
+        while let Some(bytes) = stream.next().await {
+            let bytes = bytes?;
+            file.write_all(bytes.as_ref()).await?;
+            bytes_written += bytes.len();
+
+            bar.inc(bytes.len() as u64);
+            if let Some(len) = response.content_length() {
+                bar.set_message(format!("Downloading {} ({}/{} bytes)", source.url(), bytes_written, len));
+            } else {
+                bar.set_message(format!("Downloading {} ({} bytes)", source.url(), bytes_written));
+            }
+        }
+
+        file.flush()
+            .await
+            .map_err(Error::from)
+            .map(|_| ())
+    }
+
     let force = matches.is_present("force");
     let cache = PathBuf::from(config.source_cache_root());
     let sc = SourceCache::new(cache);
@@ -233,14 +294,23 @@ pub async fn download(
         mp
     };
 
+    let matching_regexp = matches.value_of("matching")
+        .map(crate::commands::util::mk_package_name_regex)
+        .transpose()?;
+
     let r = repo
         .packages()
-        .filter(|p| pname.as_ref().map(|n| p.name() == n).unwrap_or(true))
         .filter(|p| {
-            pvers
-                .as_ref()
-                .map(|v| v.matches(p.version()))
-                .unwrap_or(true)
+            match (pname.as_ref(), pvers.as_ref(), matching_regexp.as_ref()) {
+                (None, None, None)              => true,
+                (Some(pname), None, None)       => p.name() == pname,
+                (Some(pname), Some(vers), None) => p.name() == pname && vers.matches(p.version()),
+                (None, None, Some(regex))       => regex.is_match(p.name()),
+
+                (_, _, _) => {
+                    panic!("This should not be possible, either we select packages by name and (optionally) version, or by regex.")
+                },
+            }
         })
         .map(|p| {
             sc.sources_for(p).into_iter().map(|source| {
@@ -260,57 +330,6 @@ pub async fn download(
                     if source_path_exists && !force {
                         Err(anyhow!("Source exists: {}", source.path().display()))
                     } else {
-                        async fn perform_download(source: &SourceEntry, bar: &indicatif::ProgressBar) -> Result<()> {
-                            trace!("Creating: {:?}", source);
-                            let file = source.create().await.with_context(|| {
-                                anyhow!(
-                                    "Creating source file destination: {}",
-                                    source.path().display()
-                                )
-                            })?;
-
-                            let mut file = tokio::io::BufWriter::new(file);
-                            let client = reqwest::Client::builder()
-                                .redirect(reqwest::redirect::Policy::limited(10))
-                                .build()
-                                .context("BUilding HTTP client failed")?;
-                            let request = client.get(source.url().as_ref())
-                                .build()
-                                .with_context(|| anyhow!("Building request for {} failed", source.url().as_ref()))?;
-
-                            let response = match client.execute(request).await {
-                                Ok(resp) => resp,
-                                Err(e) => {
-                                    bar.finish_with_message(format!("Failed: {}", source.url()));
-                                    return Err(e).with_context(|| anyhow!("Downloading '{}'", source.url()))
-                                }
-                            };
-
-                            if let Some(len) = response.content_length() {
-                                bar.set_length(len);
-                            }
-
-                            let mut stream = reqwest::get(source.url().as_ref()).await?.bytes_stream();
-                            let mut bytes_written = 0;
-                            while let Some(bytes) = stream.next().await {
-                                let bytes = bytes?;
-                                file.write_all(bytes.as_ref()).await?;
-                                bytes_written += bytes.len();
-
-                                bar.inc(bytes.len() as u64);
-                                if let Some(len) = response.content_length() {
-                                    bar.set_message(format!("Downloading {} ({}/{} bytes)", source.url(), bytes_written, len));
-                                } else {
-                                    bar.set_message(format!("Downloading {} ({} bytes)", source.url(), bytes_written));
-                                }
-                            }
-
-                            file.flush()
-                                .await
-                                .map_err(Error::from)
-                                .map(|_| ())
-                        }
-
                         if source_path_exists /* && force is implied by 'if' above*/ {
                             if let Err(e) = source.remove_file().await {
                                 bar.finish_with_message(format!("Failed to remove existing file: {}", source.path().display()));
