@@ -13,18 +13,19 @@ use std::path::PathBuf;
 use std::str::FromStr;
 use std::sync::Arc;
 
-use anyhow::anyhow;
 use anyhow::Context;
 use anyhow::Error;
 use anyhow::Result;
+use anyhow::anyhow;
 use futures::FutureExt;
 use getset::{CopyGetters, Getters};
 use log::trace;
+use result_inspect::ResultInspect;
 use shiplift::Container;
 use shiplift::Docker;
 use shiplift::ExecContainerOptions;
-use tokio::sync::mpsc::UnboundedSender;
 use tokio::sync::RwLock;
+use tokio::sync::mpsc::UnboundedSender;
 use tokio_stream::StreamExt;
 use typed_builder::TypedBuilder;
 
@@ -245,6 +246,7 @@ impl Endpoint {
     pub fn utilization(&self) -> f64 {
         let max_jobs = self.num_max_jobs() as f64;
         let run_jobs = self.running_jobs() as f64;
+        trace!("utilization of {}: 100.0 / {} * {}", self.name(), max_jobs, run_jobs);
         100.0 / max_jobs * run_jobs
     }
 
@@ -279,6 +281,24 @@ impl Endpoint {
             })
     }
 
+    pub async fn number_of_running_containers(&self) -> Result<usize> {
+        self.docker
+            .containers()
+            .list({
+                &shiplift::builder::ContainerListOptions::builder()
+                .all()
+                .build()
+            })
+            .await
+            .map_err(Error::from)
+            .map(|list| {
+                list.into_iter()
+                    .inspect(|stat| trace!("stat = {:?}", stat))
+                    .filter(|stat| stat.state == "running")
+                    .count()
+            })
+    }
+
     pub async fn has_container_with_id(&self, id: &str) -> Result<bool> {
         self.container_stats()
             .await?
@@ -296,6 +316,23 @@ impl Endpoint {
             Ok(None)
         }
     }
+
+    pub async fn images(&self, name_filter: Option<&str>) -> Result<impl Iterator<Item = Image>> {
+        let mut listopts = shiplift::builder::ImageListOptions::builder();
+
+        if let Some(name) = name_filter {
+            listopts.filter_name(name);
+        } else {
+            listopts.all();
+        }
+
+        self.docker
+            .images()
+            .list(&listopts.build())
+            .await
+            .map_err(Error::from)
+            .map(|v| v.into_iter().map(Image::from))
+    }
 }
 
 /// Helper type to store endpoint statistics
@@ -357,18 +394,42 @@ impl From<shiplift::rep::Container> for ContainerStat {
     }
 }
 
+#[derive(Getters)]
+pub struct Image {
+    #[getset(get = "pub")]
+    created: chrono::DateTime<chrono::Utc>,
+
+    #[getset(get = "pub")]
+    id: String,
+
+    #[getset(get = "pub")]
+    tags: Option<Vec<String>>,
+}
+
+impl From<shiplift::rep::Image> for Image {
+    fn from(img: shiplift::rep::Image) -> Self {
+        Image {
+            created: img.created,
+            id: img.id,
+            tags: img.repo_tags,
+        }
+    }
+}
+
 pub struct EndpointHandle(Arc<Endpoint>);
 
 impl EndpointHandle {
     pub fn new(ep: Arc<Endpoint>) -> Self {
-        let _ = ep.running_jobs.fetch_add(1, std::sync::atomic::Ordering::Relaxed);
+        let res = ep.running_jobs.fetch_add(1, std::sync::atomic::Ordering::Relaxed);
+        trace!("Endpoint {} has one job more: {}", ep.name(), res + 1);
         EndpointHandle(ep)
     }
 }
 
 impl Drop for EndpointHandle {
     fn drop(&mut self) {
-        let _ = self.0.running_jobs.fetch_sub(1, std::sync::atomic::Ordering::Relaxed);
+        let res = self.0.running_jobs.fetch_sub(1, std::sync::atomic::Ordering::Relaxed);
+        trace!("Endpoint {} has one job less: {}", self.0.name(), res - 1);
     }
 }
 
@@ -478,6 +539,7 @@ impl<'a> PreparedContainer<'a> {
             .containers()
             .create(&builder_opts)
             .await
+            .with_context(|| anyhow!("Creating container with builder options = {:?}", builder_opts))
             .with_context(|| anyhow!("Creating container on '{}'", endpoint.name))?;
         trace!("Create info = {:?}", create_info);
         Ok(create_info)
@@ -522,12 +584,16 @@ impl<'a> PreparedContainer<'a> {
                     .with_context(|| anyhow!("Reading file {}", source_path.display()))?;
 
                 drop(entry);
-                let _ = container.copy_file_into(destination, &buf).await?;
-                Ok(())
+                container.copy_file_into(destination, &buf)
+                    .await
+                    .inspect(|_| trace!("Successfully copied source {} to container {}", source_path.display(), container.id()))
+                    .with_context(|| anyhow!("Failed to copy source {} to container {}", source_path.display(), container.id()))
+                    .map_err(Error::from)
             })
             .collect::<futures::stream::FuturesUnordered<_>>()
             .collect::<Result<()>>()
             .await
+            .inspect(|_| trace!("Successfully copied sources to container {}", container.id()))
             .with_context(|| anyhow!("Copying sources to container {}", container.id()))
             .map_err(Error::from)
     }
@@ -560,12 +626,18 @@ impl<'a> PreparedContainer<'a> {
                     .await
                     .with_context(|| anyhow!("Reading file {}", patch.display()))?;
 
-                let _ = container.copy_file_into(destination, &buf).await?;
-                Ok(())
+                container.copy_file_into(destination, &buf)
+                    .await
+                    .map_err(Error::from)
+                    .inspect(|_| trace!("Copying patch {} successfull", patch.display()))
+                    .with_context(|| anyhow!("Copying patch {} to container {}", patch.display(), container.id()))
+                    .map_err(Error::from)
             })
             .collect::<futures::stream::FuturesUnordered<_>>()
             .collect::<Result<()>>()
             .await
+            .map_err(Error::from)
+            .inspect(|_| trace!("Copied all patches"))
             .with_context(|| anyhow!("Copying patches to container {}", container.id()))
             .map_err(Error::from)
     }
@@ -611,7 +683,10 @@ impl<'a> PreparedContainer<'a> {
                                     found = Some(path);
                                     break;
                                 },
-                                Err(e) => return Err(e),
+                                Err(e) => {
+                                    trace!("Failed to join '{:?}' + '{:?}'", release_store.root_path(), art.display());
+                                    return Err(e)
+                                },
                                 Ok(None) =>  continue,
                             }
                         }
@@ -626,10 +701,12 @@ impl<'a> PreparedContainer<'a> {
                         art.display()
                     )
                 })?;
+                trace!("Successfully read {} into buffer", art.display());
 
                 let r = container
                     .copy_file_into(&destination, &buf)
                     .await
+                    .inspect(|_| trace!("Successfully copied {} to container", art.display()))
                     .with_context(|| {
                         anyhow!(
                             "Copying artifact {} to container {} at {}",
@@ -645,6 +722,7 @@ impl<'a> PreparedContainer<'a> {
             .collect::<futures::stream::FuturesUnordered<_>>()
             .collect::<Result<Vec<_>>>()
             .await
+            .inspect(|_| trace!("Successfully copied all artifacts to the container {}", container.id()))
             .with_context(|| anyhow!("Copying artifacts to container {}", container.id()))
             .map_err(Error::from)
             .map(|_| ())
@@ -658,17 +736,18 @@ impl<'a> PreparedContainer<'a> {
         container
             .copy_file_into(script_path, script.as_ref().as_bytes())
             .await
+            .inspect(|_| trace!("Successfully copied script to container {}", container.id()))
             .with_context(|| anyhow!("Copying the script into container {}", container.id()))
             .map_err(Error::from)
     }
 
     pub async fn start(self) -> Result<StartedContainer<'a>> {
-        let container = self.endpoint.docker.containers().get(&self.create_info.id);
-        let _ = container
+        self.endpoint
+            .docker
+            .containers()
+            .get(&self.create_info.id)
             .start()
-            .inspect(|r| {
-                trace!("Starting container {} -> {:?}", self.create_info.id, r);
-            })
+            .inspect(|r| trace!("Starting container {} -> {:?}", self.create_info.id, r))
             .map(|r| {
                 r.with_context(|| {
                     anyhow!(
@@ -708,13 +787,12 @@ impl<'a> StartedContainer<'a> {
             .build();
         trace!("Exec options = {:?}", exec_opts);
 
-        let container = self.endpoint.docker.containers().get(&self.create_info.id);
-
-        trace!(
-            "Moving logs to log sink for container {}",
-            self.create_info.id
-        );
-        let stream = container.exec(&exec_opts);
+        trace!("Moving logs to log sink for container {}", self.create_info.id);
+        let stream = self.endpoint
+            .docker
+            .containers()
+            .get(&self.create_info.id)
+            .exec(&exec_opts);
 
         let exited_successfully: Option<(bool, Option<String>)> =
             buffer_stream_to_line_stream(stream)
@@ -732,7 +810,6 @@ impl<'a> StartedContainer<'a> {
                             self.create_info.id
                         )
                     })
-                    .map_err(Error::from)
                     .and_then(|l| {
                         crate::log::parser()
                             .parse(l.as_bytes())
@@ -744,31 +821,21 @@ impl<'a> StartedContainer<'a> {
                                     l
                                 )
                             })
-                            .map_err(Error::from)
-                            .and_then(|item| {
-                                let mut exited_successfully = None;
-                                {
-                                    match item {
-                                        LogItem::State(Ok(_)) => {
-                                            exited_successfully = Some((true, None))
-                                        }
-                                        LogItem::State(Err(ref msg)) => {
-                                            exited_successfully = Some((false, Some(msg.clone())))
-                                        }
-                                        _ => {
-                                            // Nothing
-                                        }
-                                    }
-                                }
-
-                                trace!("Log item: {}", item.display()?);
-                                logsink
-                                    .send(item)
-                                    .with_context(|| anyhow!("Sending log to log sink"))
-                                    .map_err(Error::from)
-                                    .map(|_| exited_successfully)
-                            })
                     })
+                    .and_then(|item| {
+                        let exited_successfully = match item {
+                            LogItem::State(Ok(_)) => Some((true, None)),
+                            LogItem::State(Err(ref msg)) => Some((false, Some(msg.clone()))),
+                            _ => None, // Nothing
+                        };
+
+                        trace!("Log item: {}", item.display()?);
+                        logsink
+                            .send(item)
+                            .with_context(|| anyhow!("Sending log to log sink"))
+                            .map(|_| exited_successfully)
+                    })
+                    .map_err(Error::from)
                 })
                 .collect::<Result<Vec<_>>>()
                 .map(|r| {
