@@ -64,14 +64,14 @@ impl StagingStore {
                 if self.0.root_path().is_dir(&path) {
                     None
                 } else {
-                    Some({
-                        // Clippy doesn't detect this properly
-                        #[allow(clippy::redundant_clone)]
-                        ArtifactPath::new(path.to_path_buf())
-                            .inspect(|r| trace!("Loaded from path {} = {:?}", path.display(), r))
-                            .with_context(|| anyhow!("Loading from path: {}", path.display()))
-                            .map(|ap| self.0.load_from_path(&ap).clone())
-                    })
+                    // Clippy doesn't detect this properly
+                    #[allow(clippy::redundant_clone)]
+                    ArtifactPath::new(path.to_path_buf())
+                        .inspect(|r| trace!("Loaded from path {} = {:?}", path.display(), r))
+                        .with_context(|| anyhow!("Loading from path: {}", path.display()))
+                        .map(|ap| self.0.load_from_path(&ap).clone())
+                        .map(Some)
+                        .transpose()
                 }
             })
             .collect()
