@@ -43,6 +43,7 @@ use crate::filestore::StagingStore;
 use crate::job::Dag;
 use crate::job::JobDefinition;
 use crate::job::RunnableJob;
+use crate::orchestrator::util::*;
 use crate::source::SourceCache;
 use crate::util::EnvironmentVariableName;
 use crate::util::progress::ProgressBars;
@@ -415,6 +416,7 @@ impl<'a> Orchestrator<'a> {
                 let sender = prep.3.into_inner().unwrap_or_else(|| vec![root_sender.clone()]);
                 JobTask::new(prep.0, prep.1, sender)
             })
+            .inspect(|task| trace!("Running: {}", task.jobdef.job.uuid()))
             .map(|task| task.run())
             .collect::<futures::stream::FuturesUnordered<_>>();
         debug!("Built {} jobs", running_jobs.len());
@@ -422,6 +424,7 @@ impl<'a> Orchestrator<'a> {
         let multibar_block = tokio::task::spawn_blocking(move || multibar.join());
         let (_, jobs_result) = tokio::join!(multibar_block, running_jobs.collect::<Result<()>>());
         let _ = jobs_result?;
+        trace!("All jobs finished");
         match root_receiver.recv().await {
             None                     => Err(anyhow!("No result received...")),
             Some(Ok(results)) => {
@@ -503,10 +506,19 @@ struct JobTask<'a> {
 impl<'a> Drop for JobTask<'a> {
     fn drop(&mut self) {
         if !self.bar.is_finished() {
-            self.bar.finish_with_message(format!("[{} {} {}] Stopped, error on other task",
+            // If there are dependencies, the error is probably from another task
+            // If there are no dependencies, the error was caused by something else
+            let errmsg = if self.jobdef.dependencies.is_empty() {
+                "error occured"
+            } else {
+                "error on other task"
+            };
+
+            self.bar.finish_with_message(format!("[{} {} {}] Stopped, {msg}",
                 self.jobdef.job.uuid(),
                 self.jobdef.job.package().name(),
-                self.jobdef.job.package().version()));
+                self.jobdef.job.package().version(),
+                msg = errmsg));
         }
     }
 }
@@ -548,12 +560,13 @@ impl<'a> JobTask<'a> {
             self.jobdef.dependencies.iter().map(|u| u.to_string()).collect::<Vec<String>>()
         });
 
+        let dep_len = self.jobdef.dependencies.len();
         // A list of job run results from dependencies that were received from the tasks for the
         // dependencies
-        let mut received_dependencies: HashMap<Uuid, Vec<ProducedArtifact>> = HashMap::new();
+        let mut received_dependencies: HashMap<Uuid, Vec<ProducedArtifact>> = HashMap::with_capacity(dep_len);
 
         // A list of errors that were received from the tasks for the dependencies
-        let mut received_errors: HashMap<Uuid, Error> = HashMap::with_capacity(self.jobdef.dependencies.len());
+        let mut received_errors: HashMap<Uuid, Error> = HashMap::with_capacity(dep_len);
 
         // Helper function to check whether all UUIDs are in a list of UUIDs
         let all_dependencies_are_in = |dependency_uuids: &[Uuid], list: &HashMap<Uuid, Vec<_>>| {
@@ -571,7 +584,7 @@ impl<'a> JobTask<'a> {
                     self.jobdef.job.package().name(),
                     self.jobdef.job.package().version(),
                     received_dependencies.iter().filter(|(rd_uuid, _)| self.jobdef.dependencies.contains(rd_uuid)).count(),
-                    self.jobdef.dependencies.len())
+                    dep_len)
             });
             trace!("[{}]: Updated bar", self.jobdef.job.uuid());
 
@@ -579,13 +592,14 @@ impl<'a> JobTask<'a> {
             // receive from the receiver
             let continue_receiving = self.perform_receive(&mut received_dependencies, &mut received_errors).await?;
 
-            trace!("[{}]: Received errors = {:?}", self.jobdef.job.uuid(), received_errors);
+            trace!("[{}]: Received errors = {}", self.jobdef.job.uuid(), received_errors.display_error_map());
             // if there are any errors from child tasks
             if !received_errors.is_empty() {
                 // send them to the parent,...
                 //
                 // We only send to one parent, because it doesn't matter
                 // And we know that we have at least one sender
+                log::error!("[{}]: Received errors = {}", self.jobdef.job.uuid(), received_errors.display_error_map());
                 self.sender[0].send(Err(received_errors)).await;
 
                 // ... and stop operation, because the whole tree will fail anyways.
@@ -627,11 +641,12 @@ impl<'a> JobTask<'a> {
                 .chain(self.git_commit_env.cloned().into_iter())
                 .collect::<Vec<_>>();
 
-            let replacement_artifacts = crate::db::find_artifacts(
-                self.database.clone(),
-                self.config,
-                self.jobdef.job.package(),
-                &self.release_stores,
+            let replacement_artifacts = crate::db::FindArtifacts::builder()
+                .database_connection(self.database.clone())
+                .config(self.config)
+                .package(self.jobdef.job.package())
+                .release_stores(&self.release_stores)
+                .image_name(Some(self.jobdef.job.image()))
 
                 // We can simply pass the staging store here, because it doesn't hurt. There are
                 // two scenarios:
@@ -646,10 +661,11 @@ impl<'a> JobTask<'a> {
                 // The fact that released artifacts are returned prefferably from this function
                 // call does not change anything, because if there is an artifact that's a released
                 // one that matches this job, we should use it anyways.
-                Some(&staging_store),
-                &additional_env,
-                true
-            )?;
+                .staging_store(Some(&staging_store))
+                .env_filter(&additional_env)
+                .script_filter(true)
+                .build()
+                .run()?;
 
             debug!("[{}]: Found {} replacement artifacts", self.jobdef.job.uuid(), replacement_artifacts.len());
             trace!("[{}]: Found replacement artifacts: {:?}", self.jobdef.job.uuid(), replacement_artifacts);
